!+
! Module: constants
!
! Authors:  Richard West - SSC (Leicester)
!          
! Description: Define useful mathematical and physical constants
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module constants

  use dal

  implicit none 

  private dummy
  
  public

  ! PI related constants
  real (kind=double), parameter :: Pi = 3.1415926535897931d0
  real (kind=double), parameter :: TwoPi = 2.0d0*PI
  real (kind=double), parameter :: FourPi = 4.0d0*PI
  real (kind=double), parameter :: PiOverTwo = 0.5d0*PI
  real (kind=double), parameter :: OneOverPi = 1.0d0/PI
  real (kind=double), parameter :: OneOverTwoPi = 1.0d0/TwoPI

  ! Angular conversion factors
  real (kind=double), parameter :: DegToRad = TwoPi/360.d0
  real (kind=double), parameter :: RadToDeg = 360.d0/TwoPi
  real (kind=double), parameter :: PiOn180 = DegToRad
  real (kind=double), parameter :: ArcminToRad = DegToRad/60.0d0
  real (kind=double), parameter :: RadToArcmin = RadToDeg*60.0d0
  real (kind=double), parameter :: ArcsecToRad = DegToRad/3600.0d0
  real (kind=double), parameter :: RadToArcsec = RadToDeg*3600.0d0

  ! Solid angle conversion factors
  real (kind=double), parameter :: SqDegToSterad = DegToRad*DegToRad
  real (kind=double), parameter :: SqArcminToSterad = SqDegToSterad/3600.0d0
  real (kind=double), parameter :: SqArcsecToSterad = SqDegToSterad/1.296d7
  real (kind=double), parameter :: SteradToSqDeg = RadToDeg*RadToDeg
  real (kind=double), parameter :: SteradToSqArcmin = SteradToSqDeg*3600.d0
  real (kind=double), parameter :: SteradToSqArcSec = SteradToSqDeg*1.296d7

  ! Square roots
  real (kind=double), parameter :: RootTwo = 1.4142135623730951d0
  real (kind=double), parameter :: RootThree = 1.7320508075688772d0
  real (kind=double), parameter :: RootTen = 3.1622776601683795d0
  real (kind=double), parameter :: RootPi = 1.7724538509055159d0

  ! Natural log
  real (kind=double), parameter :: NapierE = 2.7182818284590455d0
  real (kind=double), parameter :: lnTwo = 0.6931471805599453d0
  real (kind=double), parameter :: lnTen = 2.3025850929940459d0
  real (kind=double), parameter :: lnPI = 1.1447298858494002d0

!  ! Fundamental physical constants (from Zombeck)
!  real (kind=double), parameter :: SpeedOfLight = 299792458.0d0 ! m/sec
!  real (kind=double), parameter :: PlanckH = 6.6260755d-34 ! J sec
!  real (kind=double), parameter :: NewtonG = 6.67259d-11 ! m^3/kg/s^2

  ! Fundamental physical constants (1998 CODATA recommended)
  real (kind=double), parameter :: SpeedOfLight = 2.99792458d8    ! m/sec
  real (kind=double), parameter :: PlanckH      = 6.62606876d-34  ! J sec
  real (kind=double), parameter :: NewtonG      = 6.673d-11       ! m^3/kg/s^2
  real (kind=double), parameter :: ElectronQ    = 1.602176462d-19 ! coulombs
  real (kind=double), parameter :: Boltzmann    = 1.3806503d-23   ! J/K
  real (kind=double), parameter :: StefanBoltzmann = 5.670400d-8  ! W/m^2/K^4
  real (kind=double), parameter :: Avogadro     = 6.02214199d23   ! mol^-1

  ! Energy conversion factors
!  real (kind=double), parameter :: ErgToeV = 6.24150636309403d11
  real (kind=double), parameter :: ErgToeV = 1.0d-7 / ElectronQ
  real (kind=double), parameter :: ErgTokeV = ErgToeV/1000.0d0
  real (kind=double), parameter :: eVToErg = 1.0d0/ErgToeV
  real (kind=double), parameter :: keVToErg = 1000.0d0/ErgToeV

  ! Solar system constants
  real (kind=double), parameter :: EarthRadius = 6378.14 ! km

contains

  subroutine dummy              ! Won't link properly without
  end subroutine dummy          ! at least one subroutine entry

end module constants
