MODULE reallocate_mod

  USE errorhandling
  USE dal

  IMPLICIT NONE

  PUBLIC

  INTERFACE reallocate
     MODULE PROCEDURE realloc_real32, realloc_real64, realloc_int8, &
          realloc_int16, realloc_int32, realloc_str, realloc_bool
  END INTERFACE

CONTAINS

  SUBROUTINE realloc_real32(p, n) 

    REAL(KIND=single), POINTER, DIMENSION(:) :: p, pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_real32


  SUBROUTINE realloc_real64(p, n) 

    REAL(KIND=double), POINTER, DIMENSION(:) :: p, pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_real64


  SUBROUTINE realloc_int8(p, n) 

    INTEGER(KIND=int8), POINTER, DIMENSION(:) :: p, pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_int8


  SUBROUTINE realloc_int16(p, n) 

    INTEGER(KIND=int16), POINTER, DIMENSION(:) :: p, pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_int16


  SUBROUTINE realloc_int32(p, n) 

    INTEGER(KIND=int32), POINTER, DIMENSION(:) :: p, pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_int32


  SUBROUTINE realloc_bool(p, n) 

    LOGICAL(KIND=bool), POINTER, DIMENSION(:) :: p, pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_bool


  SUBROUTINE realloc_str(p, n) 

    CHARACTER(LEN=*), POINTER, DIMENSION(:) :: p
    CHARACTER(LEN=LEN(p)), POINTER, DIMENSION(:) :: pnew
    INTEGER, INTENT(in) :: n
    INTEGER :: nold, ierr, lb

    IF(.NOT. ASSOCIATED(p)) RETURN

    lb = LBOUND(p,1)

    ALLOCATE( pnew(lb:lb+n-1), stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error allocating space in REALLOCATE"
    ENDIF
       
    nold = MIN(SIZE(p),n) + lb - 1
    pnew(:nold) = p(:nold)

    DEALLOCATE(p, stat=ierr)
    IF( ierr /= 0 )THEN 
       WRITE(errstr, *) "Error deallocating space in REALLOCATE"
    ENDIF

    p => pnew

  END SUBROUTINE realloc_str

END MODULE reallocate_mod
